package ogloszenia.soap;

import jakarta.jws.*;
import jakarta.xml.ws.soap.MTOM;
import ogloszenia.Ustawienia;
import ogloszenia.baza.sqlite.DostepDoBazySqlite;
import ogloszenia.baza.sqlite.OgloszenieDAO;
import ogloszenia.exn.BladBazyDanych;
import ogloszenia.exn.NieznanyRekord;
import ogloszenia.model.OgloszenieSamochodowe;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@WebService(wsdlLocation = "/WEB-INF/wsdl/OgloszeniaService.wsdl",
        targetNamespace = "http://soap.ogloszenia/",
        name = "Ogloszenia",
        portName = "OgloszeniaPort",
        serviceName = "Ogloszenia")
@MTOM
@HandlerChain(file="handlers.xml")
public class Ogloszenia {
    // Gdy operacje webserwisu modyfikują wspólną strukturę danych, to trzeba zadbać o jej synchronizację.
    private final List<String> imiona = Collections.synchronizedList(new ArrayList<>());
    // dla słownika warto użyć ConcurrentHashMap

    @WebResult(name="ogloszenie")
    public List<OgloszenieSamochodowe> readAll() throws BladBazyDanych {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.readAllFull();
        }
    }

    @WebResult(name="ogloszenie")
    public OgloszenieSamochodowe readOne(@WebParam(name="id") int idOgloszenia) throws BladBazyDanych, NieznanyRekord {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.findByIdFull(idOgloszenia);
        }
    }

    @WebResult(name="ogloszenie")
    public List<OgloszenieSamochodowe> readByPrice(
            @WebParam(name="min") BigDecimal min,
            @WebParam(name="max") BigDecimal max) throws BladBazyDanych {
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.ogloszeniaWedlugCeny(min, max);
        }
    }

    public void update(
            @WebParam(name="ogloszenie") OgloszenieSamochodowe ogloszenie) throws BladBazyDanych, NieznanyRekord {
        // W tej wersji zakładamy, że obiekt ma podane id i nadpisujemy istniejące.
        // W tym przypadku nie trzeba dawać informacji zwrotnej.
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            boolean sukces = dao.update(ogloszenie);
            if(!sukces) {
                throw new NieznanyRekord("Brak ogłoszenia nr " + ogloszenie.getIdOgloszenia());
            }
            db.endTransaction(true);
        }
    }

    @WebResult(name="new-id")
    public int insert(
            @WebParam(name="ogloszenie") OgloszenieSamochodowe ogloszenie) throws BladBazyDanych {
        // W tej wersji zawsze dodajemy nowy rekord.
        // W razie, gdyby klient przysłał ogłoszenie z ustawionym id, to my i tak to id zignorujemy.
        // W tej wersji w odpowiedzi wysyłamy id nowego rekordu, aby klient dowiedział się jakie ono jest.
        ogloszenie.setIdOgloszenia(null);
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            dao.insertNew(ogloszenie);
            db.endTransaction(true);
            return ogloszenie.getIdOgloszenia();
        }
    }

    @WebResult(name="bytes")
    public byte[] photo(@WebParam(name="id") int idOgloszenia) throws NieznanyRekord {
        try {
            Path path = Paths.get(Ustawienia.PHOTOS_PATH, idOgloszenia + ".jpg");
            return Files.readAllBytes(path);
        } catch (IOException e) {
            throw new NieznanyRekord("Brak zdjęcia dla ogłoszenia nr " + idOgloszenia);
        }
    }

    public String poczekaj(
            @WebParam(name="imie") String imie,
            @WebParam(name="czas") long czas) {
        System.out.println("poczekaj początek");
        try {
            Thread.sleep(czas);
        } catch (InterruptedException e) {
        }
        System.out.println("poczekaj koniec");
        return "Koniec czekania, pozdrawiam " + imie;
    }

    @Oneway
    public void dodajImie(@WebParam(name="imie") String imie) {
        try {
            Thread.sleep(500);
            // celowo opóźniam działanie, aby pokazać różnicę w działaniu z @Oneway i bez
            // @Oneway powoduje, że po odebraniu zapytania klient i serwer rozłączają się,
            // serwer nie odsyła żadnego XML-a w odpowiedzi, a klient nie czeka na nią
        } catch (InterruptedException e) {
        }
        imiona.add(imie);
    }

    @WebResult(name="imiona")
    public String pobierzImiona() {
        synchronized(imiona) {
            return String.join(", ", imiona);
        }
    }
}
